package simplexe;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Scanner;

public class Simplexe {
    
    private double[] c;
    private double[][] A;
    private double[] b;
    private double z = 0;
    private boolean opt = false;
    private long temps_execution;
    
    public Simplexe(int n, int m) throws ConstraintsNumberException{
        if(m>n){
            throw new ConstraintsNumberException();
        }
        this.c = new double[n+m];
        this.A = new double[m][n+m];
        this.b = new double[m];
        for(int i=n; i<c.length; i++){
            c[i] = 0;
        }
        for(int l=0; l<b.length; l++){
            for(int k=n; k<c.length; k++){
                A[l][k] = (k-n == l ? 1 : 0);
            }
        }
        for(int i=0; i<n; i++){
            c[i] = rand(500);
        }
        for(int l=0; l<b.length; l++){
            for(int k=0; k<n; k++){
                A[l][k] = rand(100);
            }
        }
        for(int i=0; i<b.length; i++){
            b[i] = randBase(25, 250);
        }
        
        c[0]=3;
        c[1]=5;
        c[2]=1;

        A[0][0]=8;
        A[0][1]=16;
        A[0][2]=19;
        A[1][0]=9;
        A[1][1]=25;
        A[1][2]=7;

        b[0]=17;
        b[1]=24;
        
        ExecuteProgramme();
        
        long debut_execution = System.currentTimeMillis();
        while(!opt){
            detPivot();
        }
        temps_execution = System.currentTimeMillis()-debut_execution;
        
        writeFile(n, m, temps_execution);
    }
    
    public void ExecuteProgramme(){
        System.out.print("Programme lineaire :\n");
        System.out.print("Max : z " + (z==0 ? "":z ) + " = ");
        for(int i=0; i<c.length; i++){
            System.out.print(c[i] + " x"+ i + (i==c.length-1 ? "":" + "));
        }System.out.println();
        System.out.println("S.C. :");
        for(int l=0; l<b.length; l++){
            for(int k=0; k<c.length; k++){
                System.out.print(A[l][k] + " x"+ k + (k==c.length-1 ? /*" ⩽ "*/" = "+b[l] :" + "));
            }System.out.println();
        }
        for(int i=0; i<c.length; i++){
            System.out.print("x" + i + (i==c.length-1 ? " ⩾ 0":", "));
        }System.out.println("\n");
    }
    
    public void detPivot(){
        int idProfit = -1;
        double max = Double.MIN_VALUE;
        for(int i=0; i<c.length; i++){
            if(c[i] > 0 && c[i] > max){
                max = c[i];
                idProfit = i;
            }
        }
        if(idProfit == -1){
            opt = true;
            //////
            System.out.println("La solution est optimale : z = " + -z);
        }else{
            opt = false;
            int idPivot = regleBland(idProfit);
            if(idPivot == -1){
                System.out.println("La solution est non bornée.");
            }else{
                double sol = -pivotage(idPivot, idProfit);
                System.out.println("Nouvelle solution : " + sol + "\n");
            }
        }
    }
    
    public int regleBland(int idProfit){
        int idPivot = -1;
        double min = Double.MAX_VALUE;
        for(int i=0; i<b.length; i++){
            if(A[i][idProfit] > 0 && b[i]/A[i][idProfit]<min){
                min = b[i]/A[i][idProfit];
                idPivot = i;
            }
        }
        return idPivot;
    }
    
    public double pivotage(int idPivot, int idProfit){
        double alpha = 0;
        for(int l=0; l<b.length; l++){
            alpha = -A[l][idProfit]/A[idPivot][idProfit];
            if(l != idPivot){
                for(int k=0; k<c.length; k++){
                    A[l][k] += alpha*A[idPivot][k];
                }
                b[l] += alpha*b[idPivot];
            }
        }
        
        //Fonction objectif
        alpha = -c[idProfit]/A[idPivot][idProfit];
        for(int i=0; i<c.length; i++){
            c[i] += alpha*A[idPivot][i];
        }
        z += alpha*b[idPivot];
        
        //Ligne du pivot
        alpha = A[idPivot][idProfit];
        for(int i=0; i<c.length; i++){
            A[idPivot][i] /= alpha;
        }
        b[idPivot] /= alpha;
        return z;
    }
    
    public double rand(int borne){
        return Math.random()*borne;
    }
    
    public double randBase(int min, int max){
        return min+Math.random()*(max-min);
    }
    
    public boolean estOptimal(){
        return true;
    }
    
    public void writeFile(int n, int m, long execT) {
        BufferedWriter bw;
        try {
            File results = new File("/home/k2b/Bureau/progL_java/resultat.csv");
            if(results.exists() && !results.isDirectory()){
                Scanner sc = new Scanner(results);
                String content = "";
                int i = 0;
                while(sc.hasNextLine()){
                    content += sc.nextLine() + "," + (i==0 ? n:"") + (i==1 ? m:"") + (i==2 ? execT:"") + "\r\n";
                    i++;
                }
                sc.close();
                bw = new BufferedWriter(new FileWriter(results));
                bw.write(content);
                bw.close();
            }else{
                bw = new BufferedWriter(new FileWriter(results));
                bw.write(n +"\r\n"+ m + "\r\n"+ execT);
                bw.close();
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
